from .b2p_abc import Best2PayABC
from ...exception import ErrorPayerIDNotExist


class RedirectB2PABC(Best2PayABC):

    def do(self) -> str:
        return f'{self._get_url_b2p()}/{self.path_to_point_api}?{self._get_params()}'

    def _get_raw_signature(self):
        if self.save_card and not self.payer_id:
            raise ErrorPayerIDNotExist()

        if getattr(self, 'save_card', False):
            return str(self.sector) + str(self.id) + str(self.payer_id) + self.b2p_token
        return str(self.sector) + str(self.id) + self.b2p_token

    def _get_params(self) -> str:
        return '&'.join([f'{item}={getattr(self, item)}' for item in self.__dict__
                         if getattr(self, item) and (item not in ['is_prod_stand', 'b2p_token'])])



from abc import ABC

from ..mixin import MixinGetURLStandB2P, MixinCreateSignature


class Best2PayABC(MixinGetURLStandB2P, MixinCreateSignature, ABC):
    def do(self):
        raise NotImplementedError('Definition "do" in {self.__class__.__name__}.')



from copy import copy

import requests
import xmltodict
from requests import RequestException

from ... import exception
from ...constant import MAX_COUNT_REQUEST_B2P


class MixinRequestPostB2P:
    def _request_best2pay(self) -> dict:

        count_request_to_b2p = 1
        while True:
            try:
                res = requests.post(f'{self._get_url_b2p()}/{self.path_to_point_api}',
                                    data=self._get_date_by_send_b2p())
            except RequestException as err:
                if count_request_to_b2p < MAX_COUNT_REQUEST_B2P:
                    count_request_to_b2p += 1
                    continue
                raise exception.ErrorConnectB2P() from err
            res_dict = self._parser_xml_to_dict(res.text)
            return res_dict

    @staticmethod
    def _parser_xml_to_dict(text: str) -> dict:
        res_dict = xmltodict.parse(text, process_namespaces=True)
        return res_dict

    def _get_date_by_send_b2p(self) -> dict:
        data = copy(self.__dict__)
        data.pop('is_prod_stand')
        data.pop('b2p_token')
        return data



import base64
import hashlib


class MixinCreateSignature:
    def _create_signature(self):
        signature_raw = self._get_raw_signature()
        signature_md5 = hashlib.md5(signature_raw.encode('utf8')).digest()
        signature_md5_hex = signature_md5.hex()
        signature = base64.b64encode(signature_md5_hex.encode())
        return signature.decode()

    def _get_raw_signature(self) -> str:
        raise NotImplementedError(f'Definition "_get_raw_signature" in {self.__class__.__name__}.')



__version__ = '0.1.0'

from .wrapper_class.authorize import Authorize  # noqa: F401
from .wrapper_class.authorize_by_token import AuthorizeByToken  # noqa: F401
from .wrapper_class.authorize_by_token import AuthorizeByToken  # noqa: F401
from .wrapper_class.authorize_inc import AuthorizeInc  # noqa: F401
from .wrapper_class.complete import Complete  # noqa: F401
from .wrapper_class.operation import Operation  # noqa: F401
from .wrapper_class.order import Order  # noqa: F401
from .wrapper_class.purchase import Purchase  # noqa: F401
from .wrapper_class.purchase_by_token import PurchaseByToken  # noqa: F401
from .wrapper_class.register import Register  # noqa: F401
from .wrapper_class.reverse import Reverse  # noqa: F401



class BaseExceptionB2P(Exception):
    pass


class ErrorConnectB2P(BaseExceptionB2P):
    def __init__(self, message='B2P service not responding, please try again later'):
        self.message = message
        super().__init__(self.message)


class ErrorPayerIDNotExist(BaseExceptionB2P):
    def __init__(self, message="You must specify the payer's ID"):
        self.message = message
        super().__init__(self.message)


class ErrorDefined(BaseExceptionB2P):
    def __init__(self, message="ID or Reference defined"):
        self.message = message
        super().__init__(self.message)



from typing import Union, Optional, Any

from .class_abs import RedirectB2PABC


class Authorize(RedirectB2PABC):
    path_to_point_api = 'Authorize'

    allowed_field_with_default_value_to_b2p = {
        'get_token': 0,
        'token': False,
        'payer_id': False,
        'save_card': False,
        'is_prod_stand': False
    }

    def __init__(self, sector: str, id_: Union[str, int], b2p_token: str, **kwargs: Optional[Any]):
        self.sector = sector
        self.id = id_
        self.b2p_token = b2p_token
        for item in self.allowed_field_with_default_value_to_b2p:
            setattr(self, item, kwargs.get(item) or self.allowed_field_with_default_value_to_b2p.get(item))
        self.signature = self._create_signature()



from typing import Union, Optional, Any

from ..constant import RUB
from .class_abs import RequestB2PABC


class AuthorizeInc(RequestB2PABC):
    path_to_point_api = 'AuthorizeInc'
    allowed_field_with_default_value_to_b2p = {
        'fee': 0,
        'is_prod_stand': False
    }

    def __init__(self, sector: int, id_: Union[str, int], amount: int, b2p_token: str,
                 currency: str = RUB, **kwargs: Optional[Any]):
        self.sector = sector
        self.id = id_
        self.amount = amount
        self.currency = currency
        self.b2p_token = b2p_token
        for item in self.allowed_field_with_default_value_to_b2p:
            setattr(self, item, kwargs.get(item) or self.allowed_field_with_default_value_to_b2p.get(item))
        self.signature = self._create_signature()

    def do(self) -> dict:
        return self._request_best2pay()

    def _get_raw_signature(self) -> str:
        if hasattr(self, 'fee') and self.fee:
            return (str(self.sector) + str(self.id) + str(self.amount) +
                    str(self.fee) + str(self.currency) + self.b2p_token)
        return str(self.sector) + str(self.id) + str(self.amount) + str(self.currency) + self.b2p_token



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['wrapper_b2p',
 'wrapper_b2p.wrapper_class',
 'wrapper_b2p.wrapper_class.class_abs',
 'wrapper_b2p.wrapper_class.mixin']

package_data = \
{'': ['*']}

install_requires = \
['pytest-cov>=3.0.0,<4.0.0',
 'requests>=2.26.0,<3.0.0',
 'xmltodict>=0.12.0,<0.13.0']

setup_kwargs = {
    'name': 'wrapper-b2p',
    'version': '0.1.0',
    'description': 'wrapper best2pay',
    'long_description': None,
    'author': 'Aleksey',
    'author_email': 'ataraspost@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from ... import settings


class MixinGetURLStandB2P:
    is_prod_stand = False

    def _get_url_b2p(self) -> bool:
        if self.is_prod_stand:
            return settings.URL_PROD_STAND
        return settings.URL_TEST_STAND



import base64
import hashlib
from typing import Union, Optional, Any

from ..exception import ErrorDefined
from .class_abs import RequestB2PABC


class Order(RequestB2PABC):
    path_to_point_api = 'Order'
    allowed_field_with_default_value_to_b2p = {
        'mode': 0,
        'get_token': 0,
        'is_prod_stand': False
    }

    def __init__(self, sector: Union[str, int], b2p_token: str, id_: Union[str, int] = None,
                 reference: Union[str, int] = None, **kwargs: Optional[Any]):
        self.sector = sector
        self.id = id_
        self.reference = reference
        if not any([self.id, self.reference]):
            raise ErrorDefined()
        self.b2p_token = b2p_token
        for item in self.allowed_field_with_default_value_to_b2p:
            setattr(self, item, kwargs.get(item) or self.allowed_field_with_default_value_to_b2p.get(item))
        self.signature = self._create_signature()

    def do(self):
        return self._request_best2pay()

    def _create_signature(self):
        signature_raw = self._get_raw_signature()
        md5 = hashlib.md5(signature_raw.encode('utf8')).digest()
        md5_hex = md5.hex()
        signature = base64.b64encode(md5_hex.encode())
        return signature.decode()

    def _get_raw_signature(self) -> str:
        if self.id:
            return str(self.sector) + str(self.id) + self.b2p_token
        return str(self.sector) + str(self.reference) + self.b2p_token

    def _get_date_by_send_b2p(self) -> dict:
        data = super()._get_date_by_send_b2p()
        if self.id is None:
            data.pop('id')
        if self.reference is None:
            data.pop('reference')
        return data
